﻿using System;
using System.IO;
using System.Text;

namespace Asp.Net
{
    public abstract class FilterBase : Stream
    {
        private readonly Stream _baseStream;
        private readonly Encoding _encoding;

        public FilterBase(Stream baseStream, Encoding encoding)
        {
            _baseStream = baseStream;
            _encoding = encoding;
        }

        public override void Close()
        {
            _baseStream.Close();
        }

        public override void Flush()
        {
            if (_baseStream != null)
                _baseStream.Flush();
        }

        public override int Read(byte[] buffer, int offset, int count)
        {
            throw new NotImplementedException();
        }

        public override long Seek(long offset, SeekOrigin origin)
        {
            throw new NotImplementedException();
        }

        public override void SetLength(long value)
        {
            throw new NotImplementedException();
        }

        public override void Write(byte[] buffer, int offset, int count)
        {
            Encoding encoding = _encoding ?? Encoding.UTF8;
            string bufferString = encoding.GetString(buffer);
            string filteredString = FilterString(bufferString);
            if (filteredString == null)
                _baseStream.Write(buffer, offset, count);
            else
            {
                byte[] filteredBuffer = encoding.GetBytes(filteredString);
                _baseStream.Write(filteredBuffer, 0, filteredBuffer.Length);
            }
        }

        protected virtual string FilterString(string str)
        {
            throw new NotImplementedException();
        }

        public override bool CanRead
        {
            get
            {
                return false;
            }
        }

        public override bool CanSeek
        {
            get
            {
                return false;
            }
        }

        public override bool CanWrite
        {
            get
            {
                return _baseStream != null ? _baseStream.CanWrite : true;
            }
        }

        public override long Length
        {
            get
            {
                throw new NotImplementedException();
            }
        }

        public override long Position
        {
            get
            {
                throw new NotImplementedException();
            }
            set
            {
                throw new NotImplementedException();
            }
        }

        protected Stream BaseStream
        {
            get
            {
                return _baseStream;
            }
        }
    }
}