﻿using System;
using System.IO;
using System.Reflection;
using System.Text;
using System.Web;
using JSBuild;

namespace Asp.Net.Handlers
{
    public sealed class ResourceCombinerHandler : HttpHandlerBase
    {
        public override void ProcessRequest(HttpContext context)
        {
            StringBuilder sb = new StringBuilder();
            int index = 1;
            string assemblyName = context.Request.QueryString["asm" + index.ToString()];
            while (!string.IsNullOrEmpty(assemblyName))
            {
                string ns = context.Request.QueryString["ns" + index.ToString()] ?? assemblyName;
                string res = context.Request.QueryString["res" + index.ToString()];
                if (string.IsNullOrEmpty(res))
                    throw new Exception("No resources specified");
                string[] resources = res.Split(',');

                Assembly assembly = Assembly.Load(assemblyName);
                foreach (string resource in resources)
                {
                    string resourceName = string.Join(".", new string[] { ns, resource });
                    using (Stream resourceStream = assembly.GetManifestResourceStream(resourceName))
                    using (StreamReader reader = new StreamReader(resourceStream))
                        sb.AppendLine(reader.ReadToEnd());
                }

                index++;
                assemblyName = context.Request.QueryString["asm" + index.ToString()];
            }

            string type = context.Request.QueryString["type"];
            string contentType = string.IsNullOrEmpty(type) ? "text/plain" : "text/" + type.ToLower();

#if DEBUG
            string finalScript = sb.ToString();
#else
            string finalScript;
            if (contentType == "text/javascript")
                finalScript = new JSMin().MinifyToString(sb.ToString());
            else if (contentType == "text/css")
                finalScript = sb.ToString(); //TODO: Minify the CSS
            else
                finalScript = sb.ToString();
#endif

            string assemblyDate = context.Request.QueryString["t"];
            if (!string.IsNullOrEmpty(assemblyDate))
            {
                context.Response.Cache.SetCacheability(HttpCacheability.Public);
                context.Response.Cache.SetExpires(DateTime.Now + TimeSpan.FromDays(365));
                context.Response.Cache.SetValidUntilExpires(true);
            }

            context.Response.ContentType = contentType;
            context.Response.Write(finalScript);
        }
    }
}
