﻿using System;
using System.IO;
using System.Web;

namespace Asp.Net.Handlers
{
    public sealed class StaticFileHandler : HttpHandlerBase
    {
        public override void ProcessRequest(HttpContext context)
        {
            string filename = context.Request.QueryString["f"];
            if (filename == null)
                throw new ArgumentNullException("f", "filename query string missing for static file handler");

            string physicalFileName = context.Server.MapPath("~/" + filename);
            if (!File.Exists(physicalFileName))
                throw new InvalidOperationException(string.Format("File {0} does not exist", filename));

            string mimeType = GetMimeType(physicalFileName);
            if (mimeType == null)
                throw new ArgumentException("Invalid static file requested");

            byte[] fileBytes = File.ReadAllBytes(physicalFileName);
            context.Response.OutputStream.Write(fileBytes, 0, fileBytes.Length);
            context.Response.ContentType = mimeType;

            if (context.Request.QueryString["t"] != null)
            {
                context.Response.Cache.SetCacheability(HttpCacheability.Public);
                context.Response.Cache.SetExpires(DateTime.Now + TimeSpan.FromDays(365));
                context.Response.Cache.SetValidUntilExpires(true);
            }
        }

        private static string GetMimeType(string filename)
        {
            string extension = Path.GetExtension(filename).ToUpper();
            switch (extension)
            {
                case ".JPG":
                case ".JPEG":
                    return "image/jpg";
                case ".GIF":
                    return "image/gif";
                case ".PNG":
                    return "image/png";
                case ".CSS":
                    return "text/css";
                case ".JS":
                    return "text/javascript";
                default:
                    return null;
            }
        }
    }

}