﻿using System.Web;
using System;
using System.IO.Compression;

namespace Asp.Net.Modules
{
    public sealed class CompressionModule : FilterModuleBase
    {
        public override void Init(HttpApplication context)
        {
            context.PostRequestHandlerExecute += CompressResponse;
        }

        private void CompressResponse(object sender, EventArgs e)
        {
            HttpApplication application = (HttpApplication)sender;

            string contentType = application.Response.ContentType;
            if (!contentType.StartsWith("text/", StringComparison.OrdinalIgnoreCase))
                return;

            string acceptedEncodings = application.Request.Headers["Accept-encoding"];
            if (acceptedEncodings == null)
                return;

            if (acceptedEncodings.Contains("deflate"))
            {
                application.Response.Filter = new DeflateStream(application.Response.Filter, CompressionMode.Compress);
                application.Response.AppendHeader("Content-encoding", "deflate");
            }
            else if (acceptedEncodings.Contains("gzip"))
            {
                application.Response.Filter = new GZipStream(application.Response.Filter, CompressionMode.Compress);
                application.Response.AppendHeader("Content-encoding", "gzip");
            }
        }
    }
}
